META = {
    "type": "time-based",
    "models": {
        "DatabaseCSV": {
            "public": True,
            "any_inputs": True,
            "params": [
                "filename",
                "verbose",
                "path",
                "keep_old_files",
                "unique_filename",
            ],
            "attrs": [],
        }
    },
}
